package no.ntnu.stianren.applist;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import android.app.ActivityManager;
import android.app.ListActivity;
import android.app.NotificationManager;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PermissionInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Resources;
import android.database.Cursor;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CheckedTextView;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;


public class SelectedApp extends ListActivity {

	ListView lv = null;
	InstalledPackage pkg;
	String mockedPerm;
	
	Map<String, Permission> perms;
	NotificationManager nm;

	private static class Permission {
		private String name;
		private String label;
		private boolean mocked;
		public Permission() {
			mocked = false;
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getName() {
			return name;
		}
		public void setLabel(String label) {
			this.label = label;
		}
		public String getLabel() {
			return label;
		}
		public void setMocked(boolean mocked) {
			this.mocked = mocked;
		}
		public boolean isMocked() {
			return mocked;
		}
	}

	// A list of interesting permissions we can mock
	private static String[] INTERESTING_PERMS = {"android.permission.INTERNET",
		"android.permission.READ_SMS",
		"android.permission.WRITE_SMS",
		"android.permission.RECEIVE_SMS",
		"android.permission.RECEIVE_MMS",
		"android.permission.READ_CONTACTS",
		"android.permission.WRITE_CONTACTS",
		"android.permission.READ_CALENDAR",
		"android.permission.WRITE_CALENDAR",
		"android.permission.ACCESS_COARSE_LOCATION",
		"android.permission.ACCESS_FINE_LOCATION",
		"android.permission.MANAGE_ACCOUNTS",
		"android.permission.WRITE_EXTERNAL_STORAGE"
	};
	
	private Intent i;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		i = getIntent();
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.addmock);
		Arrays.sort(INTERESTING_PERMS);
	}


	@Override
	public void onResume() {
		super.onResume();

		String pkgName = (String) i.getExtras().get("pkg");
		Log.i(AppList.TAG, pkgName);
		mockedPerm = (String) i.getExtras().get("perm");
		
		if (mockedPerm != null) Log.i(AppList.TAG, mockedPerm);
		
		Drawable defaultIcon = getResources().getDrawable(android.R.drawable.sym_def_app_icon);
		PackageManager pm = getPackageManager();
		PackageInfo pi = null;
		perms = new LinkedHashMap<String, Permission>();
		
		try {
			pkg = new InstalledPackage(pm ,pkgName, defaultIcon);
			pi = pm.getPackageInfo(pkg.getPkg(), PackageManager.GET_PERMISSIONS | PackageManager.GET_RECEIVERS);
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}


		TextView tv = (TextView) findViewById(R.id.Label);
		tv.setText(pkg.getName());
		ImageView iv = (ImageView) findViewById(R.id.Icon);
		iv.setImageDrawable(pkg.getIcon());
		lv = (ListView) findViewById(android.R.id.list);
		nm = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

		LinkedHashSet<String> requestedPermissions;

		if (pi != null) {
			if (pi.requestedPermissions == null) {
				requestedPermissions = new LinkedHashSet<String>();
			} else {
				requestedPermissions = new LinkedHashSet<String>(Arrays.asList(pi.requestedPermissions));
			}
			if (pi.receivers != null) {
				for (ActivityInfo receiver: pi.receivers) {
					if (receiver.permission != null) {
						requestedPermissions.add(receiver.permission);
					}
				}
			}
			
			for(String permName: requestedPermissions) {
				PermissionInfo permInfo = null;
				String desc = permName;
				try {
					permInfo = pm.getPermissionInfo(permName, PackageManager.GET_META_DATA);
					desc = (String) pm.getText(permInfo.packageName, permInfo.labelRes, null);
					Log.i("AppInfo", "permInfo: " + permInfo);
					Log.i("AppInfo", "desc: " + desc);
				} catch (NameNotFoundException e) {
					e.printStackTrace();
				}
				if (Arrays.binarySearch(INTERESTING_PERMS, permName) >= 0) {
					Permission p = new Permission();
					p.setName(permName);
					p.setLabel(desc);
					perms.put(permName, p);
				}
			}
		}


		final Context c = (Context) this;

	    
		lv.setAdapter(new ArrayAdapter<Permission>(this, R.layout.row, R.id.textView1, new ArrayList<Permission>(perms.values())) {

			@Override
			public View getView(int position, View convertView, ViewGroup parent) {
				LayoutInflater vi = LayoutInflater.from(c);
				View v = vi.inflate(R.layout.row, parent, false);
				TextView tv = (TextView)v.findViewById(R.id.textView1);
				CheckBox cb = (CheckBox)v.findViewById(R.id.checkBox1);
				cb.setText("");
				Permission p = getItem(position);
				tv.setText(p.getLabel());
				
				/*
				 * Find ID of getLabel() in the Content Provider
				 */

				String IDToLabel = findId(p.getLabel());
	
				if (IDToLabel != null) 
				{
					if (checkFake(IDToLabel) == true)
					{
						cb.setChecked(true);
					}
					if (checkFake(IDToLabel) == false)
					{
						cb.setChecked(false);
					}
					//Log.i("Permissions", "ID of " + p.getLabel() + ": " + IDToLabel);
				}
				else
				{
					Log.i("Permissions", "Couldn't find ID of " + p.getLabel());
				}
				
				
				
				tv.setTextSize(15);
				v.setClickable(false);
				v.setFocusable(false);
				final int pos = position;
				tv.setOnClickListener(new View.OnClickListener() {
					
					@Override
					public void onClick(View v) {
						// TODO Auto-generated method stub
						Log.i("AppList", "TextView in row " + pos);
						Intent myIntent = new Intent(SelectedApp.this, InfoPage.class);
						SelectedApp.this.startActivity(myIntent);
						Log.i("AppList", "Just started InfoPage activity..");
					}
				});
				
				cb.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
					
					@Override
					public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
						Log.i("AppList", "Button in row: " + pos);
						if (pos == 0) {
							ContentValues editedValues = new ContentValues();
							if (isChecked == false) {
								Log.i("AppList", "Removed settings");
								editedValues.put("fake", "0");
								editedValues.put("fake_setting", "0");
								getContentResolver().update(Uri.parse("content://no.ntnu.provider.settings/permissions/" + findId("fine (GPS) location")), editedValues, null, null);
							}
							else {
								editedValues.put("fake", "1");
								editedValues.put("fake_setting", "5");
								getContentResolver().update(Uri.parse("content://no.ntnu.provider.settings/permissions/" + findId("fine (GPS) location")), editedValues, null, null);
								Log.i("AppList", "Added settings");
							}
						}
					}
				});
				
				return v;
			}
			
			
		});

		lv.invalidate();
		
			
	}



	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);
		Permission p = (Permission) l.getItemAtPosition(position);
		//CheckedTextView tv = (CheckedTextView) v.findViewById(android.R.id.text1);
		TextView tv = (TextView) v.findViewById(android.R.id.text1);
		p.setMocked(!p.isMocked());
/*
		if (!p.isMocked()) {
			tv.setTextColor(0xffdf6926);
			tv.setChecked(false);
		} else {
			tv.setTextColor(0xff26df6f);
			tv.setChecked(true);
		}
	*/
		
		
		/*
		if (p.getName().equals("android.permission.INTERNET")) {
			ActivityManager m = (ActivityManager) getSystemService(ACTIVITY_SERVICE);
			m.restartPackage(pkg.getName());
		}
		*/

		perms.put(p.getName(), p);
		Log.i("AppInfo", "onListItemClick position: " + position);
	}
	
	public String findId(String toFind) 
	{
		Uri allEntries = Uri.parse("content://no.ntnu.provider.settings/permissions");
	    Cursor c2 = managedQuery(allEntries, null, null, null, null);
	    String s = null;
	    String retur = null;
	    if (c2.moveToFirst())
	    {
	    	do
	     	{
	    		s = c2.getString(c2.getColumnIndex("AP"));
	     		if (s.equals(toFind))
	     		{
	     			//Log.i("Permissions", "Found it!");
	     			retur = c2.getString(c2.getColumnIndex("_id"));
	     			break;
	     		}
	     	}
	    	while (c2.moveToNext());
	    }
	    return retur;
	}
	
	public boolean checkFake(String id)
	{
		Uri allEntries = Uri.parse("content://no.ntnu.provider.settings/settings");
		Cursor c = managedQuery(allEntries, null, null, null, null);
		String s = null;
		boolean retur = false;
		if (c.moveToFirst())
		{
			do
			{
				s = c.getString(c.getColumnIndex("_id"));
				if (s.equals(id))
				{
					if (c.getString(c.getColumnIndex("fake")).equals("1"))
					{
						retur = true;
					}
					else
					{
						retur = false;
					}
					//Log.i("Permissions", "Modified access for id " + id + ": " + c.getString(c.getColumnIndex("fake")));
				}
			} while (c.moveToNext());
		}
		return retur;
	}

}

